#========================================================================================
# package      DE_EPAGES::Product::t::ProductHandler
#----------------------------------------------------------------------------------------
# description  Test TLEs processing related to Products
#========================================================================================
package DE_EPAGES::Product::t::ProductHandler;
use base DE_EPAGES::Shop::API::Test;

## no critic (Tics::ProhibitLongLines)

use strict;
use Readonly;
use Test::More tests => 20;

use DE_EPAGES::Product::API::TLE::ProductHandler;
use DE_EPAGES::TLE::API::BaseProcessor;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

sub GetSubProduct($$) {
    my ($MasterProduct, $SubProductAlias) = @_;
    return $MasterProduct->child('SubProducts')->child($SubProductAlias);
};

sub DisplayPriceMsg {
    my (%Strings) = @_;
    return sprintf 'DISPLAYPRICE - SubProductAlias=%s IsDefault=%s HasSubOwnPrices=%s DontShowFrom=%s',
        $Strings{Alias}, $Strings{IsDefault}, $Strings{HasOwnPrice}, $Strings{DontShowFrom};
}

sub testProductURL {
    my $self = shift;

    my $Shop = $self->shop;
    my $SuperProduct = $Shop->path('Products/MyJacket');
    is( $SuperProduct->get('DefaultSubProduct')->alias, 'JacketRed2', 'SuperProduct.DefaultSubProduct' );

    my $Processor = DE_EPAGES::TLE::API::BaseProcessor->new;
    my $ProductHandler = DE_EPAGES::Product::API::TLE::ProductHandler->new();
    $ProductHandler->register($Processor);

    my $ProductURL = sub {
        my ($Object, $SearchParams) = @_;
        local $Processor->{'INPUT'} = {};
        $Processor->{'INPUT'}->{'SearchParams'} = $SearchParams if defined $SearchParams;
        $Processor->callBlock('ProductURL', sub {[ $Object ]});
    };

    is( $ProductURL->($SuperProduct),
        '?ObjectPath=/Test/TestShop/Products/MyJacket/SubProducts/JacketRed2',
        'ProductURL(SuperProduct) in category' );
    is( $ProductURL->($SuperProduct, '{"Color":"black"}'),
        '?ObjectPath=/Test/TestShop/Products/MyJacket&ViewAction=ViewProduct&SearchParams={"Color":"black"}',
        'ProductURL(SuperProduct) in search results with filter' );

    my $SubProduct = $SuperProduct->child('SubProducts')->child('JacketBlack3');
    is( $ProductURL->($SubProduct),
        '?ObjectPath=/Test/TestShop/Products/MyJacket/SubProducts/JacketBlack3',
        'ProductURL(SubProduct) in category' );
    is( $ProductURL->($SubProduct, '{"Color":"black"}'),
        '?ObjectPath=/Test/TestShop/Products/MyJacket/SubProducts/JacketBlack3&ViewAction=ViewProduct&SearchParams={"Color":"black"}',
        'ProductURL(SubProduct) in search results with filter' );

    note 'testing ProductHandler TLEs';

    Readonly my $CurrencyID => 'EUR';
    Readonly my $ProductPrice => 16.8;
    Readonly my $TAX_NETTO => 0;
    my $Product = $self->productFolder->child('TestProduct');
    #HASPRICE($Product, $CurrencyID, $User)
    is( $ProductHandler->HASPRICE( $ProductHandler, [ $Product, $CurrencyID, $self->testUser ] ), 0, 'HASPRICE no price' );
    $Product->setListPrice( $CurrencyID, $TAX_NETTO, $ProductPrice );
    is( $ProductHandler->HASPRICE( $ProductHandler, [ $Product, $CurrencyID, $self->testUser ] ), 1, 'HASPRICE price exists' );

    ##############################
    # Normal Product
    ##############################

    #DISPLAYPRICE($Product, $CurrencyID, $TaxModel, $User, $NoFromPrice)
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $Shop->get('LanguageID');
    my $DontShowFromPrice = 'yes';

    is(
        $ProductHandler->DISPLAYPRICE(
            $ProductHandler, [ $Product, $CurrencyID, $TAX_NETTO, $self->testUser, $DontShowFromPrice ]
        ),
        $ProductPrice,
        'DISPLAYPRICE standard product costs ' . $ProductPrice
    );
    $DontShowFromPrice = undef;
    is(
        $ProductHandler->DISPLAYPRICE(
            $ProductHandler, [ $Product, $CurrencyID, $TAX_NETTO, $self->testUser, $DontShowFromPrice ]
        ),
        $ProductPrice,
        'DISPLAYPRICE standard product "from" price eq list price'
    );
    $Product->setListPrice( $CurrencyID, undef, undef );
    is(
        $ProductHandler->DISPLAYPRICE(
            $ProductHandler, [ $Product, $CurrencyID, $TAX_NETTO, $self->testUser, 0 ]
        ),
        undef,
        'DISPLAYPRICE standard product has no price'
    );

    ####################################################################################################################
    #
    #    DISPLAYPRICE with VariationProducts
    #
    #    This is rich an possibillities -- we only check 2**4 possible combination and focus on flags
    #    IsDefault, HasOwnPrice, DontShowFrom tested with default objects SubProduct and std. SubProduct objects
    #
    #    Note: Prices are "gross" values, but in XML you see only "net" values.
    #
    ####################################################################################################################

    #
    # FROM PRICE
    #
    note 'testing VariationProducts -- show "from" price aka always the cheapest price';
    ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
    Readonly my $DefaultSubAlias => 'JacketRed2';
    Readonly my $SubAlias => 'JacketBlack2';
    my $ShowFromPrice = 0;

    # MasterProduct
    is(
        $ProductHandler->DISPLAYPRICE(
            $ProductHandler, [$SuperProduct, $CurrencyID, $TAX_NETTO, $self->testUser, $ShowFromPrice ]
        ),
        135.62,
        'DISPLAYPRICE MasterProduct show "from" price'
    );

    # SubProduct with own price
    is(
        $ProductHandler->DISPLAYPRICE(
            $ProductHandler, [GetSubProduct($SuperProduct, $SubAlias), $CurrencyID, $TAX_NETTO, $self->testUser, $ShowFromPrice ]
        ),
        135.62,
        DisplayPriceMsg( IsDefault => 0, Alias => $SubAlias, HasOwnPrice => 1, DontShowFrom => $ShowFromPrice)
    );

    # SubProduct with default price
    GetSubProduct($SuperProduct, $SubAlias)->set({HasSubOwnPrices => 0});
    is(
        $ProductHandler->DISPLAYPRICE(
            $ProductHandler, [GetSubProduct($SuperProduct, $SubAlias), $CurrencyID, $TAX_NETTO, $self->testUser, $ShowFromPrice ]
        ),
        150.62,
        DisplayPriceMsg( IsDefault => 0, Alias => $SubAlias, HasOwnPrice => 0, DontShowFrom => $ShowFromPrice)
    );
    GetSubProduct($SuperProduct, $SubAlias)->set({HasSubOwnPrices => 1});

    # Default SubProduct with own price
    is(
        $ProductHandler->DISPLAYPRICE(
            $ProductHandler, [GetSubProduct($SuperProduct, $DefaultSubAlias), $CurrencyID, $TAX_NETTO, $self->testUser, $ShowFromPrice ]
        ),
        135.62,
        DisplayPriceMsg( IsDefault => 1, Alias => $DefaultSubAlias, HasOwnPrice => 1, DontShowFrom => $ShowFromPrice)
    );

    # Default SubProduct with default price
    GetSubProduct($SuperProduct, $DefaultSubAlias)->set({HasSubOwnPrices => 0});
    is(
        $ProductHandler->DISPLAYPRICE(
            $ProductHandler, [GetSubProduct($SuperProduct, $DefaultSubAlias), $CurrencyID, $TAX_NETTO, $self->testUser, $ShowFromPrice ]
        ),
        135.62,
        DisplayPriceMsg( IsDefault => 1, Alias => $DefaultSubAlias, HasOwnPrice => 0, DontShowFrom => $ShowFromPrice)
    );
    GetSubProduct($SuperProduct, $DefaultSubAlias)->set({HasSubOwnPrices => 1});

    #
    # NO FROM PRICE
    #
    note 'testing VariationProducts -- display price';
    $DontShowFromPrice = 1;

    # MasterProduct
    is(
        $ProductHandler->DISPLAYPRICE(
            $ProductHandler, [$SuperProduct, $CurrencyID, $TAX_NETTO, $self->testUser, $DontShowFromPrice ]
        ),
        168.03,
        'DISPLAYPRICE MasterProduct show master price'
    );

    # SubProduct with own price
    is(
        $ProductHandler->DISPLAYPRICE(
            $ProductHandler, [GetSubProduct($SuperProduct, $SubAlias), $CurrencyID, $TAX_NETTO, $self->testUser, $DontShowFromPrice ]
        ),
        135.62,
        DisplayPriceMsg( IsDefault => 0, Alias => $SubAlias, HasOwnPrice => 1, DontShowFrom => $DontShowFromPrice)
    );

    # SubProduct with default price
    GetSubProduct($SuperProduct, $SubAlias)->set({HasSubOwnPrices => 0});
    is(
        $ProductHandler->DISPLAYPRICE(
            $ProductHandler, [GetSubProduct($SuperProduct, $SubAlias), $CurrencyID, $TAX_NETTO, $self->testUser, $DontShowFromPrice ]
        ),
        168.03,
        DisplayPriceMsg( IsDefault => 0, Alias => $SubAlias, HasOwnPrice => 0, DontShowFrom => $DontShowFromPrice)
    );
    GetSubProduct($SuperProduct, $SubAlias)->set({HasSubOwnPrices => 1});

    # Default SubProduct with own price
    is(
        $ProductHandler->DISPLAYPRICE(
            $ProductHandler, [GetSubProduct($SuperProduct, $DefaultSubAlias), $CurrencyID, $TAX_NETTO, $self->testUser, $DontShowFromPrice ]
        ),
        159.62,
        DisplayPriceMsg( IsDefault => 1, Alias => $DefaultSubAlias, HasOwnPrice => 1, DontShowFrom => $DontShowFromPrice)
    );

    # Default SubProduct with default price
    GetSubProduct($SuperProduct, $DefaultSubAlias)->set({HasSubOwnPrices => 0});
    is(
        $ProductHandler->DISPLAYPRICE(
            $ProductHandler, [GetSubProduct($SuperProduct, $DefaultSubAlias), $CurrencyID, $TAX_NETTO, $self->testUser, $DontShowFromPrice ]
        ),
        168.03,
        DisplayPriceMsg( IsDefault => 1, Alias => $DefaultSubAlias, HasOwnPrice => 0, DontShowFrom => $DontShowFromPrice)
    );
    GetSubProduct($SuperProduct, $DefaultSubAlias)->set({HasSubOwnPrices => 1});
    ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
}

sub init {
    my $self = shift;

    $self->SUPER::init;
    die 'Shop alias is not TestShop' unless $self->shop->alias eq 'TestShop';

    note 'importing ProductHandler.xml';
    ImportObjects('ProductHandler.xml', $self->shop->parent->pathString);
}

sub test {
    my $self = shift;

    $self->testProductURL;
}

sub testUser {
    shift->shop->child('Users')->child('mmustermann');        # DE_EPAGES::Permission::API::Object::User
}

sub productFolder {
    shift->shop->get('ProductFolder');
}

__PACKAGE__->runScript;
